@@ -35,20 +35,43 @@ from _pygit2 import *
 # High level API
 from .repository import Repository
 from .version import __version__
-import pygit2.utils
 
 
 def init_repository(path, bare=False):
     """
-    Creates a new Git repository in the given path.
+    Creates a new Git repository in the given *path*.
 
-    Arguments:
-
-    path
-      Path where to create the repository.
-
-    bare
-      Whether the repository will be bare or not.
+    If *bare* is True the repository will be bare, i.e. it will not have a
+    working copy.
     """
     _pygit2.init_repository(path, bare)
     return Repository(path)
+
+
+def clone_repository(
+        url, path, bare=False, ignore_cert_errors=False,
+        remote_name="origin", checkout_branch=None):
+    """
+    Clones a new Git repository from *url* in the given *path*.
+
+    **bare** indicates whether a bare git repository should be created.
+
+    **remote_name** is the name given to the "origin" remote.
+    The default is "origin".
+
+    **checkout_branch** gives the name of the branch to checkout.
+    None means use the remote's *HEAD*.
+
+    Returns a Repository class pointing to the newly cloned repository.
+
+    If you wish to use the repo, you need to do a checkout for one of
+    the available branches, like this:
+
+        >>> repo = repo.clone_repository("url", "path")
+        >>> repo.checkout(branch)  # i.e.: refs/heads/master
+
+    """
+
+    _pygit2.clone_repository(
+        url, path, bare, ignore_cert_errors, remote_name, checkout_branch)
+    return Repository(path)
