@@ -43,7 +43,7 @@ extern PyTypeObject HunkType;
 PyTypeObject PatchType;
 
 PyObject*
-wrap_diff(git_diff_list *diff, Repository *repo)
+wrap_diff(git_diff *diff, Repository *repo)
 {
     Diff *py_diff;
 
@@ -58,59 +58,62 @@ wrap_diff(git_diff_list *diff, Repository *repo)
 }
 
 PyObject*
-diff_get_patch_byindex(git_diff_list* list, size_t idx)
+diff_get_patch_byindex(git_diff* diff, size_t idx)
 {
     const git_diff_delta* delta;
-    const git_diff_range* range;
-    git_diff_patch* patch = NULL;
-    size_t i, j, hunk_amounts, lines_in_hunk, line_len, header_len;
-    const char* line, *header;
-    char line_origin;
+    const git_diff_hunk *hunk;
+    const git_diff_line *line;
+    git_patch* patch = NULL;
+    size_t i, j, hunk_amounts, lines_in_hunk, additions, deletions;
     int err;
     Hunk *py_hunk = NULL;
     Patch *py_patch = NULL;
     PyObject *py_line_origin=NULL, *py_line=NULL;
 
-    err = git_diff_get_patch(&patch, &delta, list, idx);
+    err = git_patch_from_diff(&patch, diff, idx);
     if (err < 0)
         return Error_set(err);
 
+    delta = git_patch_get_delta(patch);
+
     py_patch = PyObject_New(Patch, &PatchType);
     if (py_patch != NULL) {
         py_patch->old_file_path = delta->old_file.path;
         py_patch->new_file_path = delta->new_file.path;
         py_patch->status = git_diff_status_char(delta->status);
         py_patch->similarity = delta->similarity;
+        py_patch->flags = delta->flags;
         py_patch->old_oid = git_oid_allocfmt(&delta->old_file.oid);
         py_patch->new_oid = git_oid_allocfmt(&delta->new_file.oid);
 
+        git_patch_line_stats(NULL, &additions, &deletions, patch);
+        py_patch->additions = additions;
+        py_patch->deletions = deletions;
 
-        hunk_amounts = git_diff_patch_num_hunks(patch);
+        hunk_amounts = git_patch_num_hunks(patch);
         py_patch->hunks = PyList_New(hunk_amounts);
         for (i=0; i < hunk_amounts; ++i) {
-            err = git_diff_patch_get_hunk(&range, &header, &header_len,
-                      &lines_in_hunk, patch, i);
+            err = git_patch_get_hunk(&hunk, &lines_in_hunk, patch, i);
 
             if (err < 0)
                 goto cleanup;
 
             py_hunk = PyObject_New(Hunk, &HunkType);
             if (py_hunk != NULL) {
-                py_hunk->old_start = range->old_start;
-                py_hunk->old_lines = range->old_lines;
-                py_hunk->new_start = range->new_start;
-                py_hunk->new_lines = range->new_lines;
+                py_hunk->old_start = hunk->old_start;
+                py_hunk->old_lines = hunk->old_lines;
+                py_hunk->new_start = hunk->new_start;
+                py_hunk->new_lines = hunk->new_lines;
 
                 py_hunk->lines = PyList_New(lines_in_hunk);
                 for (j=0; j < lines_in_hunk; ++j) {
-                    err = git_diff_patch_get_line_in_hunk(&line_origin,
-                              &line, &line_len, NULL, NULL, patch, i, j);
+                    err = git_patch_get_line_in_hunk(&line, patch, i, j);
 
                     if (err < 0)
-                      goto cleanup;
+                        goto cleanup;
 
-                    py_line_origin = to_unicode_n(&line_origin, 1, NULL, NULL);
-                    py_line = to_unicode_n(line, line_len, NULL, NULL);
+                    py_line_origin = to_unicode_n(&line->origin, 1, NULL, NULL);
+                    py_line = to_unicode_n(line->content, line->content_len, NULL, NULL);
                     PyList_SetItem(py_hunk->lines, j,
                         Py_BuildValue("OO",
                             py_line_origin,
@@ -128,7 +131,7 @@ diff_get_patch_byindex(git_diff_list* list, size_t idx)
     }
 
 cleanup:
-    git_diff_patch_free(patch);
+    git_patch_free(patch);
 
     return (err < 0) ? Error_set(err) : (PyObject*) py_patch;
 }
@@ -152,6 +155,24 @@ PyMemberDef Patch_members[] = {
     MEMBER(Patch, status, T_CHAR, "status"),
     MEMBER(Patch, similarity, T_INT, "similarity"),
     MEMBER(Patch, hunks, T_OBJECT, "hunks"),
+    MEMBER(Patch, additions, T_INT, "additions"),
+    MEMBER(Patch, deletions, T_INT, "deletions"),
+    {NULL}
+};
+
+PyDoc_STRVAR(Patch_is_binary__doc__, "True if binary data, False if not.");
+
+PyObject *
+Patch_is_binary__get__(Patch *self)
+{
+    if (!(self->flags & GIT_DIFF_FLAG_NOT_BINARY) &&
+            (self->flags & GIT_DIFF_FLAG_BINARY))
+        Py_RETURN_TRUE;
+    Py_RETURN_FALSE;
+}
+
+PyGetSetDef Patch_getseters[] = {
+    GETTER(Patch, is_binary),
     {NULL}
 };
 
@@ -187,7 +208,7 @@ PyTypeObject PatchType = {
     0,                                         /* tp_iternext       */
     0,                                         /* tp_methods        */
     Patch_members,                             /* tp_members        */
-    0,                                         /* tp_getset         */
+    Patch_getseters,                           /* tp_getset         */
     0,                                         /* tp_base           */
     0,                                         /* tp_dict           */
     0,                                         /* tp_descr_get      */
@@ -249,14 +270,19 @@ PyTypeObject DiffIterType = {
     (iternextfunc) DiffIter_iternext,          /* tp_iternext       */
 };
 
+Py_ssize_t
+Diff_len(Diff *self)
+{
+    assert(self->list);
+    return (Py_ssize_t)git_diff_num_deltas(self->list);
+}
 
 PyDoc_STRVAR(Diff_patch__doc__, "Patch diff string.");
 
 PyObject *
 Diff_patch__get__(Diff *self)
 {
-    const git_diff_delta* delta;
-    git_diff_patch* patch;
+    git_patch* patch;
     char **strings = NULL;
     char *buffer = NULL;
     int err = GIT_ERROR;
@@ -267,16 +293,16 @@ Diff_patch__get__(Diff *self)
     MALLOC(strings, num * sizeof(char*), cleanup);
 
     for (i = 0, len = 1; i < num ; ++i) {
-        err = git_diff_get_patch(&patch, &delta, self->list, i);
+        err = git_patch_from_diff(&patch, self->list, i);
         if (err < 0)
             goto cleanup;
 
-        err = git_diff_patch_to_str(&(strings[i]), patch);
+        err = git_patch_to_str(&(strings[i]), patch);
         if (err < 0)
             goto cleanup;
 
         len += strlen(strings[i]);
-        git_diff_patch_free(patch);
+        git_patch_free(patch);
     }
 
     CALLOC(buffer, (len + 1), sizeof(char), cleanup);
@@ -433,7 +459,7 @@ Diff_getitem(Diff *self, PyObject *value)
 static void
 Diff_dealloc(Diff *self)
 {
-    git_diff_list_free(self->list);
+    git_diff_free(self->list);
     Py_CLEAR(self->repo);
     PyObject_Del(self);
 }
@@ -444,7 +470,7 @@ PyGetSetDef Diff_getseters[] = {
 };
 
 PyMappingMethods Diff_as_mapping = {
-    0,                               /* mp_length */
+    (lenfunc)Diff_len,               /* mp_length */
     (binaryfunc)Diff_getitem,        /* mp_subscript */
     0,                               /* mp_ass_subscript */
 };
